from django.contrib import admin

from .models import HubOauthToken, HubUser, UserCompany

# Register your models here.
@admin.register(UserCompany)
class UserCompanyAdmin(admin.ModelAdmin):
    model = UserCompany
    list_display = ['uuid', 'name']

@admin.register(HubUser)
class HubUserAdmin(admin.ModelAdmin):
    model = HubUser
    list_display = ['user', 'token']

@admin.register(HubOauthToken)
class HubOauthTokenAdmin(admin.ModelAdmin):
    model = HubOauthToken
    list_display = ['uuid', 'expires_in_dt']



from django.urls import path

app_name = 'django_hub_sdk'

from .views import LoginView, CallbackView, UserMeView, LogoutView, RefreshTokenView

urlpatterns = [
    path("auth/login", LoginView.as_view(), name="login"),
    path("auth/logout", LogoutView.as_view(), name="logout"),
    path("auth/callback", CallbackView.as_view(), name="callback"),
    path("auth/refresh", RefreshTokenView.as_view(), name="refresh"),
    path("users/me", UserMeView.as_view(), name="me")
]



# Generated by Django 3.1.8 on 2021-10-19 13:31

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='HubOauthToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, verbose_name='User UUID')),
                ('access_token', models.TextField(blank=True, null=True, verbose_name='Access Token')),
                ('refresh_token', models.TextField(blank=True, null=True, verbose_name='Refresh Token')),
                ('expires_in', models.CharField(blank=True, max_length=255, null=True, verbose_name='Expires In')),
                ('expires_in_dt', models.DateTimeField(blank=True, max_length=255, null=True, verbose_name='Expires In DT')),
            ],
            options={
                'verbose_name': 'Hub Oauth Token',
                'verbose_name_plural': 'Hub Oauth Tokens',
                'db_table': 'hub_oauth_token',
            },
        ),
        migrations.CreateModel(
            name='UserCompany',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(blank=True, null=True, verbose_name='Hub Company UUID')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Hub Company Name')),
            ],
            options={
                'verbose_name': 'User Company',
                'verbose_name_plural': 'User Companys',
                'db_table': 'hub_companies',
            },
        ),
        migrations.CreateModel(
            name='HubUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='user', to='django_hub_sdk.huboauthtoken', verbose_name='Tokens')),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='tokens', to=settings.AUTH_USER_MODEL, verbose_name='Users')),
            ],
            options={
                'verbose_name': 'Hub User',
                'verbose_name_plural': 'Hub Users',
                'db_table': 'hub_users',
            },
        ),
    ]



import uuid

from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext as _


# Create your models here.

class UserCompany(models.Model):
    uuid = models.UUIDField("Hub Company UUID", null=True, blank=True)
    name = models.CharField("Hub Company Name", null=True, blank=True, max_length=255)

    class Meta:
        verbose_name = _("User Company")
        verbose_name_plural = _("User Companys")
        db_table = "hub_companies"

    def __str__(self):
        return self.name


class BaseHubUser(models.Model):
    hub_uuid = models.UUIDField("Hub User UUID", null=True, blank=True)
    avatar = models.TextField("Avatar", blank=True, null=True)
    company = models.ForeignKey(
        UserCompany, on_delete=models.CASCADE, related_name="users", null=True
    )

    class Meta:
        abstract = True


class HubOauthToken(models.Model):
    uuid = models.UUIDField(_("User UUID"), editable=False, default=uuid.uuid4)

    access_token = models.TextField(
        "Access Token", blank=True, null=True
    )
    refresh_token = models.TextField(
        "Refresh Token", blank=True, null=True
    )
    expires_in = models.CharField("Expires In", blank=True, null=True, max_length=255)
    expires_in_dt = models.DateTimeField(
        "Expires In DT", blank=True, null=True, max_length=255
    )

    def is_expired(self):
        diff_dates = timezone.now() - self.expires_in_dt
        if abs(diff_dates.days) <= 1:
            return True
        return False

    def __str__(self):
        return "{} {}".format(self.uuid, self.expires_in_dt)

    class Meta:
        verbose_name = _("Hub Oauth Token")
        verbose_name_plural = _("Hub Oauth Tokens")
        db_table = "hub_oauth_token"


class HubUser(models.Model):
    token = models.ForeignKey(
        HubOauthToken,
        verbose_name=_("Tokens"),
        on_delete=models.CASCADE,
        related_name="user",
        null=True
    )

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_("Users"), on_delete=models.CASCADE, related_name="tokens", null=True
    )

    class Meta:
        verbose_name = _("Hub User")
        verbose_name_plural = _("Hub Users")
        db_table = "hub_users"



import os

HUB_APP_SLUG = ""

HUB_BASE_URI = os.environ.get("HUB_BASE_URI", "")
HUB_BASE_PREFIX = os.environ.get("HUB_BASE_PREFIX", "/api")
HUB_BASE_FRONT_URI = os.environ.get(
    "HUB_BASE_FRONT_URI", ""
)

HUB_PROGRAMMATIC_PREFIX = os.environ.get("HUB_PROGRAMMATIC_PREFIX", "/programmatic")
HUB_PROGRAMMATIC_CLIENT = os.environ.get("HUB_PROGRAMMATIC_CLIENT", "")
HUB_PROGRAMMATIC_SECRET = os.environ.get("HUB_PROGRAMMATIC_SECRET", "")

HUB_OAUTH_CLIENT_ID = os.environ.get("HUB_OAUTH_CLIENT_ID", "")
HUB_OAUTH_CLIENT_SECRET = os.environ.get("HUB_OAUTH_CLIENT_SECRET", "")
HUB_OAUTH_REDIRECT = os.environ.get("HUB_OAUTH_REDIRECT", "")
HUB_OAUTH_SCOPES = os.environ.get("HUB_OAUTH_SCOPES", "profile")
HUB_OAUTH_AUTHORIZE_URI = os.environ.get("HUB_OAUTH_AUTHORIZE_URI", "/auth/authorize")
HUB_OAUTH_TOKEN_URI = os.environ.get("HUB_OAUTH_TOKEN_URI", "/oauth/token")
HUB_OAUTH_USERINFO_URI = os.environ.get("HUB_OAUTH_USERINFO_URI", "/users/me")



import hashlib
import random
import string
from urllib.parse import urlencode, quote_plus

import requests
from django.conf import settings
from django.core.cache import cache
from django.utils import timezone
from django.utils.translation import gettext as _
from rest_framework import views, response, status

from .authentication import HubJWTAuthentication
from .models import HubOauthToken, HubUser


# Create your views here.
class LoginView(views.APIView):
    def get(self, request):
        characters = string.ascii_letters + string.digits
        state = ''.join(random.choice(characters) for i in range(40))

        cache.set("state-" + state, state)
        cache.set(state, request.get_full_path())

        payload = {
            'client_id': settings.HUB_OAUTH_CLIENT_ID,
            'redirect_uri': settings.HUB_OAUTH_REDIRECT,
            'response_type': 'code',
            'scope': settings.HUB_OAUTH_SCOPES,
            'state': state,
            'url': request.query_params.get('url', '/')
        }

        redirect_uri = settings.HUB_BASE_FRONT_URI + settings.HUB_OAUTH_AUTHORIZE_URI
        redirect_uri += '?' + urlencode(payload, quote_via=quote_plus)

        return response.Response(data={
            "login_url": redirect_uri
        })


class CallbackView(views.APIView):
    def get(self, request):
        params = request.query_params

        code = params.get('code', '')
        state = params.get('state', '')
        url = params.get('url', '')

        cache_state = cache.get("state-" + state, '')

        if len(cache_state) > 0 and state != cache_state:
            return response.Response({
                'message': _('State is not valid')
            }, status.HTTP_404_NOT_FOUND)

        token_uri = settings.HUB_BASE_URI + settings.HUB_OAUTH_TOKEN_URI
        response_token = requests.post(token_uri, data={
            'grant_type': 'authorization_code',
            'client_id': settings.HUB_OAUTH_CLIENT_ID,
            'client_secret': settings.HUB_OAUTH_CLIENT_SECRET,
            'redirect_uri': settings.HUB_OAUTH_REDIRECT,
            'code': code,
        }, headers={
            "Accept": "application/json"
        })

        json_cache = response_token.json()
        json_cache['expires_in_dt'] = timezone.now() + timezone.timedelta(seconds=json_cache['expires_in'])

        HubOauthToken.objects.create(
            access_token=hashlib.md5(json_cache['access_token'].encode('utf-8')).hexdigest(),
            refresh_token=json_cache['refresh_token'],
            expires_in=json_cache['expires_in'],
            expires_in_dt=json_cache['expires_in_dt']
        )

        json_cache['redirect'] = url

        return response.Response(json_cache)


class UserMeView(views.APIView):
    authentication_classes = [HubJWTAuthentication]

    def get(self, request):
        url = settings.HUB_BASE_URI + settings.HUB_BASE_PREFIX + settings.HUB_OAUTH_USERINFO_URI
        query_params = {
            "project": settings.HUB_APP_SLUG
        }
        url += '?' + urlencode(query_params, quote_via=quote_plus)
        hub_response = requests.get(url, headers={
            "Accept": "application/json",
            "Authorization": request.headers.get('Authorization')
        })
        return response.Response(data=hub_response.json())


class LogoutView(views.APIView):
    authentication_classes = [HubJWTAuthentication]

    def get(self, request):
        logout_url = settings.HUB_BASE_FRONT_URI + '/auth/logout'

        hub_users = HubUser.objects.filter(user_id=request.user.id)
        hub_user_ids = hub_users.select_related('token').values_list('token_id', flat=True)
        HubOauthToken.objects.filter(id__in=hub_user_ids).delete()
        hub_users.delete()

        return response.Response(data={
            "logout_url": logout_url
        })


class RefreshTokenView(views.APIView):
    authentication_classes = [HubJWTAuthentication]

    def get(self, request):
        bearer_token = request.headers.get('Authorization')
        bearer_token_hash = hashlib.md5(bearer_token.encode('utf-8')).hexdigest()
        bearer_token_cache = HubOauthToken.objects.filter(access_token=bearer_token_hash).first()

        if bearer_token_cache is None:
            return response.Response(data={
                "status": status.HTTP_401_UNAUTHORIZED,
                "message": "Bearer Token is Required"
            }, status=status.HTTP_401_UNAUTHORIZED)

        if bearer_token_cache.is_expired():
            refresh_uri = settings.HUB_BASE_URI + settings.HUB_OAUTH_TOKEN_URI
            refresh_response = requests.post(refresh_uri, data={
                'grant_type': 'refresh_token',
                'refresh_token': bearer_token_cache.refresh_token,
                'client_id': settings.HUB_OAUTH_CLIENT_ID,
                'client_secret': settings.HUB_OAUTH_CLIENT_SECRET,
                'scope': settings.HUB_OAUTH_SCOPES,
            })
            json_cache = refresh_response.json()
            json_cache['expires_in_dt'] = timezone.now() + timezone.timedelta(seconds=json_cache['expires_in'])
            HubOauthToken.objects.create(
                access_token=hashlib.md5(json_cache['access_token'].encode('utf-8')).hexdigest(),
                refresh_token=json_cache['refresh_token'],
                expires_in=json_cache['expires_in'],
                expires_in_dt=json_cache['expires_in_dt']
            )
            return response.Response(data={
                'access_token': json_cache['access_token']
            })

        return response.Response(data={
            'access_token': bearer_token_cache.access_token
        })



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_hub_sdk', 'django_hub_sdk.migrations']

package_data = \
{'': ['*']}

install_requires = \
['django>=3.0,<4.0', 'djangorestframework', 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'django-hub-sdk',
    'version': '0.1.12',
    'description': 'The ISS (International Space Station) aims to be a space station (client) of connection between the microservices of its ecosystem and the authentication and permissions microservice of the user that here is called in the script as Hub.permissions modules / microservices (Hub)',
    'long_description': '# django-hub-sdk\n\n![PyPI](https://img.shields.io/pypi/v/django-hub-sdk)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/django-hub-sdk)\n[![Django Hub SDK](https://github.com/bildvitta/django-hub-sdk/actions/workflows/main.yml/badge.svg)](https://github.com/bildvitta/django-hub-sdk/actions/workflows/main.yml)\n\n## What Is This?\n\nThe ISS (International Space Station) aims to be a space station (client) of connection between the microservices of its\necosystem and the authentication and permissions microservice of the user that here is called in the script as\nHub.permissions modules / microservices (Hub)\n\n## How To Use This\n\n### Installation\n\n```shell\npip install django-hub-sdk\n```\n\nor if you are using Pipenv or Poetry\n\n```shell\npipenv install django-hub-sdk\n```\n\n```shell\npoetry add django-hub-sdk\n```\n\n### Settings\n\nAdd the app to installed apps\n\n```python\nINSTALLED_APPS = [\n    ...,\n    "django_hub_sdk",\n    ...,\n]\n```\n\nAll settings can be modified through environment variables, so if you are using `.env` please use it.\n\n**Mandatory Settings**\n\n```python\nimport os \n\nHUB_APP_SLUG = "" # project name on the hub\n\nHUB_BASE_URI = os.environ.get("HUB_BASE_URI", "") # HUB backend url\nHUB_BASE_FRONT_URI = os.environ.get(\n    "HUB_BASE_FRONT_URI", "" # HUB frontend url\n)\n\nHUB_PROGRAMMATIC_CLIENT = os.environ.get("HUB_PROGRAMMATIC_CLIENT", "") # Programmatic user access public key\nHUB_PROGRAMMATIC_SECRET = os.environ.get("HUB_PROGRAMMATIC_SECRET", "") # Programmatic user access private key\n\nHUB_OAUTH_CLIENT_ID = os.environ.get("HUB_OAUTH_CLIENT_ID", "") # Public oauth access key\nHUB_OAUTH_CLIENT_SECRET = os.environ.get("HUB_OAUTH_CLIENT_SECRET", "") # Private oauth access key\nHUB_OAUTH_REDIRECT = os.environ.get("HUB_OAUTH_REDIRECT", "") # oauth access return URL\n```\n\n### User Model\n\nTo continue the installation, in your User model extend the class below to be able to create the necessary relationships with the HUB\n\n```python\nfrom django.contrib.auth.models import AbstractUser\n\nfrom django_hub_sdk.models import BaseHubUser # <- Import BaseHubUser\n\n\n# Create your models here.\nclass User(AbstractUser, BaseHubUser): # <- Extends this on model\n    ...\n```\n\n### Urls\n\nAdd this line to your urls.py to work with the frontend SDK\n\n```python\nfrom django.urls import path, include\n\nurlpatterns = [\n    path("api/", include("django_hub_sdk.urls", namespace="django_hub_sdk"))\n]\n```\n\n## Use of Authorization\n\nUse of authorizations to use the JWT generated by the Hub\n\n```python\nfrom django_hub_sdk.authentication import HubJWTAuthentication # Import from package\n\nfrom rest_framework import views\n\nclass LogoutView(views.APIView):\n    authentication_classes = [HubJWTAuthentication] # <- Use on authentication_classes\n```\n\n',
    'author': 'Guilherme Haynes Howe',
    'author_email': 'zerossb@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bildvitta/django-hub-sdk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import hashlib
import random
import string
from urllib.parse import urlencode, quote_plus

import requests
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.core.cache import cache
from rest_framework import authentication

from .models import HubUser, HubOauthToken, UserCompany


class LoginUserByCache:
    def get_programmatic_jwt(self):
        url = settings.HUB_BASE_URI + settings.HUB_OAUTH_TOKEN_URI
        response = requests.post(url, headers={
            "Accept": "application/json",
        }, data={
            'grant_type': 'client_credentials',
            'client_id': settings.HUB_PROGRAMMATIC_CLIENT,
            'client_secret': settings.HUB_PROGRAMMATIC_SECRET,
            'scope': '*',
        })
        return response.json()['access_token']

    def update_or_create_company(self, user, company_uuid):
        url = settings.HUB_BASE_URI + settings.HUB_BASE_PREFIX + settings.HUB_PROGRAMMATIC_PREFIX + '/companies'
        url += '/' + company_uuid
        programmatic_token = self.get_programmatic_jwt()

        company_response = requests.get(url, headers={
            "Accept": "application/json",
            "Authorization": "Bearer " + programmatic_token
        })

        hub_company = company_response.json()['result']

        company, updated = UserCompany.objects.update_or_create(
            uuid=hub_company['uuid'],
            defaults={
                'name': hub_company['name'],
            }
        )

        user.company = company
        user.save()

    def get_hub_user(self, key):
        url = settings.HUB_BASE_URI + settings.HUB_BASE_PREFIX + settings.HUB_OAUTH_USERINFO_URI
        query_params = {
            "project": settings.HUB_APP_SLUG
        }
        url += '?' + urlencode(query_params, quote_via=quote_plus)
        response = requests.get(url, headers={
            "Accept": "application/json",
            "Authorization": "Bearer " + key
        })
        return response.json()['result']

    def update_or_create_user(self, hub_user):
        user_model = get_user_model()

        is_superuser = False
        if "is_superuser" in hub_user:
            is_superuser = True

        first_name, last_name = hub_user['name'].split(" ", 1)
        username = hub_user['email'].split('@')[0]

        user, updated = user_model.objects.update_or_create(
            email=hub_user['email'],
            hub_uuid=hub_user['uuid'],
            defaults={
                'first_name': first_name,
                'last_name': last_name,
                'is_superuser': is_superuser,
                'username': username,
                'avatar': hub_user['avatar']
            }
        )

        characters = string.ascii_letters + string.digits
        password = ''.join(random.choice(characters) for i in range(20))
        user.set_password(password)
        user.save()

        self.update_or_create_company(user, hub_user['company'])
        self.update_or_create_permissions(user, hub_user['user_permissions'])

        return user

    def update_or_create_permissions(self, user, hub_permissions):
        for key, value in hub_permissions:
            model, action = key.split(".")
            content_type = ContentType.objects.filter(model=model).first()
            if content_type:
                permission, updated = Permission.objects.update_or_create(
                    codename=action,
                    content_type=content_type,
                    defaults={
                        "name": f"{model} {action}"
                    }
                )
                user.user_permissions.add(permission)
            else:
                print(f"Não existe permissão para {model} {action}")
            user.save()


class HubJWTAuthentication(authentication.TokenAuthentication, LoginUserByCache):
    keyword = 'Bearer'

    def authenticate_credentials(self, key):
        cache_key = hashlib.md5(key.encode('utf-8')).hexdigest()
        token = HubUser.objects.filter(token__access_token=str(cache_key)).first()
        if token:
            cache.set(
                'access_token_user_id_' + str(cache_key),
                token.user_id
            )
            user_model = get_user_model()
            user = user_model.objects.get(id=token.user_id)
        else:
            hub_user = self.get_hub_user(key)
            user = self.update_or_create_user(hub_user)
            HubUser.objects.create(
                token=HubOauthToken.objects.filter(access_token=str(cache_key)).first(),
                user=user
            )

        return user, key
